@@ -181,43 +181,28 @@ gnome_custom_view_cmd (GtkWidget *widget, WPanel *panel)
 	do_refresh ();
 }
 static void
-option_menu_sort_callback(GtkWidget *widget, gpointer data)
+sort_callback (GtkWidget *menu_item, GtkWidget *cbox1)
 {
-	g_print ("FIXME: implement this function;option_menu_sort_callback\n");
-	switch ((gint) data) {
-	case SORT_NAME:
-
-		break;
-	case SORT_EXTENSION:
-		
-		break;
-	case SORT_ACCESS:
-		
-		break;
-	case SORT_MODIFY:
-		
-		break;
-	case SORT_CHANGE:
-		
-		break;
-	case SORT_SIZE:
-		
-		break;
-	}
+	if (gtk_object_get_data (GTK_OBJECT (menu_item), "SORT_ORDER_CODE") == SORT_NAME)
+		gtk_widget_set_sensitive (cbox1, TRUE);
+	else
+		gtk_widget_set_sensitive (cbox1, FALSE);
 }
 void
 gnome_sort_cmd (GtkWidget *widget, WPanel *panel)
-/* Helps you determine the order in which the files exist. */
 {
 	GtkWidget *sort_box;
 	GtkWidget *hbox;
         GtkWidget *omenu;
         GtkWidget *menu;
         GtkWidget *menu_item;
-	GtkWidget *cbox;
+	GtkWidget *cbox1, *cbox2;
+	sortfn *sfn;
 
-	sort_box = gnome_dialog_new ("Sort By", GNOME_STOCK_BUTTON_OK, 
+	sort_box = gnome_dialog_new (_("Sort By"), GNOME_STOCK_BUTTON_OK, 
 				     GNOME_STOCK_BUTTON_CANCEL, NULL);
+	/* we define this up here so we can pass it in to our callback */
+	cbox1 = gtk_check_button_new_with_label (N_("Ignore case sensitivity."));
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (sort_box)->vbox), hbox, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new ("Sort files by "), FALSE, FALSE, 0);
@@ -226,43 +211,155 @@ gnome_sort_cmd (GtkWidget *widget, WPanel *panel)
         gtk_box_pack_start (GTK_BOX (hbox), omenu, FALSE, FALSE, 0);
         menu = gtk_menu_new ();
         menu_item = gtk_menu_item_new_with_label ( _("Name"));
+	/* FIXME: we want to set the option menu to be the correct ordering. */
         gtk_menu_append (GTK_MENU (menu), menu_item);
-        gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-                            GTK_SIGNAL_FUNC (option_menu_sort_callback), (gpointer) SORT_NAME);
+	gtk_object_set_data (GTK_OBJECT (menu_item), "SORT_ORDER_CODE", (gpointer) SORT_NAME);
+	gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
+				   GTK_SIGNAL_FUNC (sort_callback), cbox1);
+	
         menu_item = gtk_menu_item_new_with_label ( _("File Type"));
         gtk_menu_append (GTK_MENU (menu), menu_item);
-        gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-                            GTK_SIGNAL_FUNC (option_menu_sort_callback), (gpointer) SORT_EXTENSION);
+	gtk_object_set_data (GTK_OBJECT (menu_item), "SORT_ORDER_CODE", (gpointer) SORT_EXTENSION);
+	gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
+				   GTK_SIGNAL_FUNC (sort_callback), cbox1);
+
 	menu_item = gtk_menu_item_new_with_label ( _("Size"));
         gtk_menu_append (GTK_MENU (menu), menu_item);
-        gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-                            GTK_SIGNAL_FUNC (option_menu_sort_callback), (gpointer) SORT_SIZE);
+	gtk_object_set_data (GTK_OBJECT (menu_item), "SORT_ORDER_CODE", (gpointer) SORT_SIZE);
+	gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
+				   GTK_SIGNAL_FUNC (sort_callback), cbox1);
+
         menu_item = gtk_menu_item_new_with_label ( _("Time Last Accessed"));
         gtk_menu_append (GTK_MENU (menu), menu_item);
-        gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-                            GTK_SIGNAL_FUNC (option_menu_sort_callback), (gpointer) SORT_ACCESS);
+	gtk_object_set_data (GTK_OBJECT (menu_item), "SORT_ORDER_CODE", (gpointer) SORT_ACCESS);
+	gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
+				   GTK_SIGNAL_FUNC (sort_callback), cbox1);
+
         menu_item = gtk_menu_item_new_with_label ( _("Time Last Modified"));
         gtk_menu_append (GTK_MENU (menu), menu_item);
-        gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-                            GTK_SIGNAL_FUNC (option_menu_sort_callback), (gpointer) SORT_MODIFY);
+	gtk_object_set_data (GTK_OBJECT (menu_item), "SORT_ORDER_CODE", (gpointer) SORT_MODIFY);
+	gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
+				   GTK_SIGNAL_FUNC (sort_callback), cbox1);
+
         menu_item = gtk_menu_item_new_with_label ( _("Time Last Changed"));
         gtk_menu_append (GTK_MENU (menu), menu_item);
-        gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-                            GTK_SIGNAL_FUNC (option_menu_sort_callback), (gpointer) SORT_CHANGE);
+	gtk_object_set_data (GTK_OBJECT (menu_item), "SORT_ORDER_CODE", (gpointer) SORT_CHANGE);
+	gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
+				   GTK_SIGNAL_FUNC (sort_callback), cbox1);
+
 	gtk_widget_show_all (menu);
         gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
 
 	
-	cbox = gtk_check_button_new_with_label (N_("Reverse the order."));
+	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (cbox1), panel->case_sensitive);
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (sort_box)->vbox),
-			    cbox, FALSE, FALSE, 0);
-	cbox = gtk_check_button_new_with_label (N_("Ignore case sensitivity."));
+			    cbox1, FALSE, FALSE, 0);
+
+	cbox2 = gtk_check_button_new_with_label (N_("Reverse the order."));
+	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (cbox2), panel->reverse);
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (sort_box)->vbox),
-			    cbox, FALSE, FALSE, 0);
+			    cbox2, FALSE, FALSE, 0);
 	/* off to the races */
 	gtk_widget_show_all (GNOME_DIALOG (sort_box)->vbox);
-	gnome_dialog_run (GNOME_DIALOG (sort_box));
+	switch (gnome_dialog_run (GNOME_DIALOG (sort_box))) {
+	case 0:
+		switch( (gint) gtk_object_get_data (GTK_OBJECT
+						    (GTK_OPTION_MENU
+						     (omenu)->menu_item),
+						    "SORT_ORDER_CODE") ) {
+		case SORT_NAME:
+			sfn = sort_name;
+			break;
+		case SORT_EXTENSION:
+			sfn = sort_ext;
+			break;
+		case SORT_ACCESS:
+			sfn = sort_atime;
+			break;
+		case SORT_MODIFY:
+			sfn = sort_time;
+			break;
+		case SORT_CHANGE:
+			sfn = sort_ctime;
+			break;
+		case SORT_SIZE:
+			sfn = sort_size;
+			break;
+		}
+		/* case sensitive */
+		panel->case_sensitive = GTK_TOGGLE_BUTTON (cbox1)->active;
+		/* Reverse order */
+		panel->reverse = GTK_TOGGLE_BUTTON (cbox2)->active;
 
-	/* Reverse order */
-	/* case sensitive */
+		panel_set_sort_order (panel, sfn);
+		break;
+	case 1:
+	default:
+		break;
+	}
+	gtk_widget_destroy (sort_box);
+}
+void
+gnome_external_panelize (GtkWidget *widget, WPanel *panel)
+{
+	GtkWidget *ep_dlg;
+	GtkWidget *frame;
+	GtkWidget *clist;
+	GtkWidget *sw;
+	GtkWidget *vbox;
+	GtkWidget *hbox;
+	GtkWidget *button;
+	GtkWidget *entry;
+
+	ep_dlg = gnome_dialog_new (_("Run Command"), GNOME_STOCK_BUTTON_OK, 
+				   GNOME_STOCK_BUTTON_CANCEL, NULL);
+
+				/* Frame 1 */
+	frame = gtk_frame_new (_("Preset Commands"));
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (ep_dlg)->vbox),
+			    frame, FALSE, FALSE, 0);
+	clist = gtk_clist_new (1);
+	gtk_clist_set_column_auto_resize (GTK_CLIST (clist), 1, TRUE);
+	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
+	sw = gtk_scrolled_window_new (GTK_CLIST (clist)->hadjustment, GTK_CLIST (clist)->vadjustment);
+	gtk_widget_set_usize (sw, 300, 100);
+	gtk_container_add (GTK_CONTAINER (sw), clist);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_box_pack_start (GTK_BOX (vbox), sw, TRUE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (frame), vbox);
+	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+	button = gtk_button_new_with_label (_("Add"));
+	gtk_widget_set_usize (button, 75, 25);
+	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	button = gtk_button_new_with_label (_("Remove"));
+	gtk_widget_set_usize (button, 75, 25);
+	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+
+				/* Frame 2 */
+	frame = gtk_frame_new (_("Run this Command"));
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (ep_dlg)->vbox),
+			    frame, FALSE, FALSE, 0);
+	entry = gtk_entry_new ();
+	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("Command: ")), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (frame), hbox);
+	gtk_widget_show_all (GNOME_DIALOG (ep_dlg)->vbox);
+	gnome_dialog_run_and_close (GNOME_DIALOG (ep_dlg));
+
+}
+void
+gnome_select_all_cmd (GtkWidget *widget, WPanel *panel)
+{
+	gint i;
+	for (i = 0; i < panel->count; i++){
+		if (!strcmp (panel->dir.list [i].fname, ".."))
+			continue;
+		do_file_mark (panel, i, 1);
+	}
+	paint_panel (panel);
+	do_refresh ();
 }
