@@ -35,6 +35,7 @@
 #define DIR_H_INCLUDE_HANDLE_DIRENT /* bleah */
 #include "dir.h"
 #include "file.h"
+#include "gconf.h"
 #include "gdesktop.h"
 #include "gdesktop-icon.h"
 #include "gicon.h"
@@ -76,7 +77,7 @@ static struct layout_slot *layout_slots;
 #define l_slots(u, v) (layout_slots[(u) * layout_rows + (v)])
 
 /* The last icon to be selected */
-static struct desktop_icon_info *last_selected_icon;
+static DesktopIconInfo *last_selected_icon;
 
 /* Drag and drop sources and targets */
 
@@ -129,7 +130,7 @@ static int click_current_y;
 static int click_dragging;
 
 
-static struct desktop_icon_info *desktop_icon_info_new (char *filename, int auto_pos, int xpos, int ypos);
+static DesktopIconInfo *desktop_icon_info_new (char *filename, int auto_pos, int xpos, int ypos);
 
 
 /* Looks for a free slot in the layout_slots array and returns the coordinates that coorespond to
@@ -205,7 +206,7 @@ get_icon_snap_pos (int *x, int *y)
 
 /* Removes an icon from the slot it is in, if any */
 static void
-remove_from_slot (struct desktop_icon_info *dii)
+remove_from_slot (DesktopIconInfo *dii)
 {
 	if (dii->slot == -1)
 		return;
@@ -224,13 +225,13 @@ remove_from_slot (struct desktop_icon_info *dii)
  * desktop_snap_icons flag is set.
  */
 static void
-desktop_icon_info_place (struct desktop_icon_info *dii, int auto_pos, int xpos, int ypos)
+desktop_icon_info_place (DesktopIconInfo *dii, int auto_pos, int xpos, int ypos)
 {
 	int u, v;
 	char *filename;
 
 	if (auto_pos) {
-		if (desktop_auto_placement)
+		if (desktop_auto_placement || !desktop_snap_icons)
 			get_icon_auto_pos (&xpos, &ypos);
 		else if (desktop_snap_icons)
 			get_icon_snap_pos (&xpos, &ypos);
@@ -270,75 +271,75 @@ desktop_icon_info_place (struct desktop_icon_info *dii, int auto_pos, int xpos,
 	g_free (filename);
 }
 
-/*
- * Returns TRUE if there is already an icon in the desktop for the
- * specified filename, FALSE otherwise.
- */
-static int
-icon_exists (char *filename)
+/* Destroys all the current desktop icons */
+static void
+destroy_desktop_icons (void)
 {
 	int i;
 	GList *l;
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 
-	for (i = 0; i < (layout_cols * layout_rows); i++)
-		for (l = layout_slots[i].icons; l; l = l->next) {
+	for (i = 0; i < (layout_cols * layout_rows); i++) {
+		l = layout_slots[i].icons;
+
+		while (l) {
 			dii = l->data;
-			if (strcmp (filename, dii->filename) == 0)
-				return TRUE;
+			l = l->next;
+
+			desktop_icon_info_destroy (dii);
 		}
-
-	return FALSE;
-}
-
-static GList *
-icon_exists_in_list (GList *list, char *filename)
-{
-	GList *l;
-
-	for (l = list; l; l = l->next){
-		struct desktop_icon_info *dii = l->data;
-
-		if (strcmp (filename, dii->filename) == 0)
-			return l;
 	}
-	return NULL;
 }
 
-/*
- * Returns a GList with all of the icons on the desktop
- */
-GList *
-desktop_get_all_icons (void)
+/* Returns a list with all of the icons on the desktop */
+static GList *
+get_all_icons (void)
 {
 	GList *l, *res;
 	int i;
 
 	res = NULL;
+
 	for (i = 0; i < (layout_cols * layout_rows); i++)
-		for (l = layout_slots [i].icons; l; l = l->next){
+		for (l = layout_slots [i].icons; l; l = l->next)
 			res = g_list_prepend (res, l->data);
-		}
 
 	return res;
 }
 
-/*
- * Reads the ~/desktop directory and creates the desktop icons.  If
- * incremental is TRUE, then an icon will not be created for a file if
- * there is already an icon for it, and icons will be created starting
- * at the specified position.
+/* Returns the node in the list of DesktopIconInfo structures that contains the
+ * icon for the specified filename.  If no icon is found, then returns NULL.
+ */
+static GList *
+icon_exists_in_list (GList *list, char *filename)
+{
+	GList *l;
+	DesktopIconInfo *dii;
+
+	for (l = list; l; l = l->next) {
+		dii = l->data;
+
+		if (strcmp (filename, dii->filename) == 0)
+			return l;
+	}
+
+	return NULL;
+}
+
+/* Reloads the desktop icons efficiently.  If there are "new" files for which no
+ * icons have been created, then icons for them will be created started at the
+ * specified position -- this is used when dragging new icons to the desktop.
  */
 static void
-load_desktop_icons (int incremental, int xpos, int ypos)
+reload_desktop_icons (int xpos, int ypos)
 {
 	struct dirent *dirent;
 	DIR *dir;
 	char *full_name;
 	int have_pos, x, y;
-	struct desktop_icon_info *dii;
-	GSList *need_position_list, *l;
-	GList *all_icons;
+	DesktopIconInfo *dii;
+	GSList *need_position_list, *sl;
+	GList *all_icons, *l;
 
 	dir = mc_opendir (desktop_directory);
 	if (!dir) {
@@ -349,35 +350,30 @@ load_desktop_icons (int incremental, int xpos, int ypos)
 		return;
 	}
 
-	/*
-	 * First create the icons for all the files that do have their
-	 * icon position set.  Build a list of the icons that do not
-	 * have their position set.
+	/* Read the directory.  For each file for which we do have an existing
+	 * icon, do nothing.  Otherwise, if the file has its metadata for icon
+	 * position set, create an icon for it.  Otherwise, store it in a list
+	 * of new icons for which positioning is pending.
 	 */
 
 	need_position_list = NULL;
-
-	all_icons = desktop_get_all_icons ();
+	all_icons = get_all_icons ();
 	
 	while ((dirent = mc_readdir (dir)) != NULL) {
-		if (((dirent->d_name[0] == '.') && (dirent->d_name[1] == 0))
-		    || ((dirent->d_name[0] == '.') && (dirent->d_name[1] == '.') && (dirent->d_name[2] == 0)))
+		/* Skip . and .. */
+
+		if ((dirent->d_name[0] == '.' && dirent->d_name[1] == 0)
+		    || (dirent->d_name[0] == '.' && dirent->d_name[1] == '.'
+			&& dirent->d_name[2] == 0))
 			continue;
 
-		if (incremental){
-			GList *element;
-
-			element = icon_exists_in_list (all_icons, dirent->d_name);
-			
-			if (element){
-				g_list_remove_link (all_icons, element);
-				continue;
-			}
-
+		l = icon_exists_in_list (all_icons, dirent->d_name);
+		if (l) {
+			all_icons = g_list_remove_link (all_icons, l);
+			continue;
 		}
-		
-		full_name = g_concat_dir_and_file (desktop_directory, dirent->d_name);
 
+		full_name = g_concat_dir_and_file (desktop_directory, dirent->d_name);
 		have_pos = gmeta_get_icon_pos (full_name, &x, &y);
 
 		if (have_pos) {
@@ -386,84 +382,54 @@ load_desktop_icons (int incremental, int xpos, int ypos)
 
 			g_free (full_name);
 		} else
-			need_position_list = g_slist_prepend (need_position_list, g_strdup (dirent->d_name));
+			need_position_list = g_slist_prepend (need_position_list,
+							      g_strdup (dirent->d_name));
 	}
 
 	mc_closedir (dir);
 
-	/*
-	 * all_icons now contains a list of all of the icons that were not found
-	 * in the ~/desktop directory, remove them.
+	/* all_icons now contains a list of all of the icons that were not found
+	 * in the ~/desktop directory, remove them.  To be paranoically tidy,
+	 * also delete the icon position information -- someone may have deleted
+	 * a file under us.
 	 */
-	if (incremental){
-		GList *l;
-		
-		for (l = all_icons; l; l = l->next){
-			struct desktop_icon_info *dii = l->data;
 
-			desktop_icon_destroy (dii);
-		}
+	for (l = all_icons; l; l = l->next) {
+		dii = l->data;
+		full_name = g_concat_dir_and_file (desktop_directory, dii->filename);
+		gmeta_del_icon_pos (full_name);
+		g_free (full_name);
+		desktop_icon_info_destroy (dii);
 	}
+
 	g_list_free (all_icons);
 
-	/*
-	 * Now create the icons for all the files that did not have their position set.  This makes
-	 * auto-placement work correctly without overlapping icons.
+	/* Now create the icons for all the files that did not have their
+	 * position set.  This makes auto-placement work correctly without
+	 * overlapping icons.
 	 */
 
 	need_position_list = g_slist_reverse (need_position_list);
 
-	for (l = need_position_list; l; l = l->next) {
-		dii = desktop_icon_info_new (l->data, TRUE, xpos, ypos);
+	for (sl = need_position_list; sl; sl = sl->next) {
+		dii = desktop_icon_info_new (sl->data, TRUE, xpos, ypos);
 		gtk_widget_show (dii->dicon);
-		g_free (l->data);
+		g_free (sl->data);
 	}
 
 	g_slist_free (need_position_list);
-}
 
-/* Destroys all the current desktop icons */
-static void
-destroy_desktop_icons (void)
-{
-	int i;
-	GList *l;
-	struct desktop_icon_info *dii;
-
-	for (i = 0; i < (layout_cols * layout_rows); i++) {
-		l = layout_slots[i].icons;
-
-		while (l) {
-			dii = l->data;
-			l = l->next;
-
-			desktop_icon_destroy (dii);
-		}
-	}
-}
-
-/*
- * Reloads the desktop icons.  If incremental is TRUE, then the
- * existing icons will not be destroyed first, and the new icons will
- * be put at the specified position.
- */
-static void
-reload_desktop_icons (int incremental, int x, int y)
-{
-	if (!incremental)
-		destroy_desktop_icons ();
-
-	load_desktop_icons (incremental, x, y);
+	/* Flush events to make the icons paint themselves */
 	x_flush_events ();
 }
 
 /* Unselects all the desktop icons except the one in exclude */
 static void
-unselect_all (struct desktop_icon_info *exclude)
+unselect_all (DesktopIconInfo *exclude)
 {
 	int i;
 	GList *l;
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 
 	for (i = 0; i < (layout_cols * layout_rows); i++)
 		for (l = layout_slots[i].icons; l; l = l->next) {
@@ -480,16 +446,16 @@ unselect_all (struct desktop_icon_info *exclude)
  * last_selected_icon and ends at the specified icon.
  */
 static void
-select_range (struct desktop_icon_info *dii, int sel)
+select_range (DesktopIconInfo *dii, int sel)
 {
 	int du, dv, lu, lv;
 	int min_u, min_v;
 	int max_u, max_v;
 	int u, v;
 	GList *l;
-	struct desktop_icon_info *ldii;
-	struct desktop_icon_info *min_udii, *min_vdii;
-	struct desktop_icon_info *max_udii, *max_vdii;
+	DesktopIconInfo *ldii;
+	DesktopIconInfo *min_udii, *min_vdii;
+	DesktopIconInfo *max_udii, *max_vdii;
 
 	/* Find out the selection range */
 
@@ -509,8 +475,17 @@ select_range (struct desktop_icon_info *dii, int sel)
 	} else {
 		min_u = lu;
 		max_u = du;
-		min_udii = last_selected_icon;
-		max_udii = dii;
+
+		/* Even if the icons are on the same slot, their positions may
+		 * need adjusting with respect to each other.
+		 */
+		if (du != lu || last_selected_icon->x < dii->x) {
+			min_udii = last_selected_icon;
+			max_udii = dii;
+		} else {
+			min_udii = dii;
+			max_udii = last_selected_icon;
+		}
 	}
 
 	if (dv < lv) {
@@ -521,8 +496,15 @@ select_range (struct desktop_icon_info *dii, int sel)
 	} else {
 		min_v = lv;
 		max_v = dv;
-		min_vdii = last_selected_icon;
-		max_vdii = dii;
+
+		/* Same as above */
+		if (dv != lv || last_selected_icon->y < dii->y) {
+			min_vdii = last_selected_icon;
+			max_vdii = dii;
+		} else {
+			min_vdii = dii;
+			max_vdii = last_selected_icon;
+		}
 	}
 
 	/* Select all the icons in the rectangle */
@@ -548,7 +530,7 @@ select_range (struct desktop_icon_info *dii, int sel)
  * event_state is the state field of the event.
  */
 static void
-select_icon (struct desktop_icon_info *dii, int event_state)
+select_icon (DesktopIconInfo *dii, int event_state)
 {
 	int range;
 	int additive;
@@ -626,7 +608,7 @@ file_entry_free (file_entry *fe)
 static int
 text_changed (GnomeIconTextItem *iti, gpointer data)
 {
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 	char *new_name;
 	char *source;
 	char *dest;
@@ -659,7 +641,7 @@ text_changed (GnomeIconTextItem *iti, gpointer data)
 static void
 editing_started (GnomeIconTextItem *iti, gpointer data)
 {
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 	GdkCursor *ibeam;
 
 	dii = data;
@@ -690,7 +672,7 @@ editing_started (GnomeIconTextItem *iti, gpointer data)
 
 /* Sets up the specified icon as a drag source, but does not connect the signals */
 static void
-setup_icon_dnd_actions (struct desktop_icon_info *dii)
+setup_icon_dnd_actions (DesktopIconInfo *dii)
 {
 	gtk_drag_source_set (DESKTOP_ICON (dii->dicon)->canvas,
 			     GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
@@ -706,7 +688,7 @@ setup_icon_dnd_actions (struct desktop_icon_info *dii)
 static void
 editing_stopped (GnomeIconTextItem *iti, gpointer data)
 {
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 
 	dii = data;
 
@@ -719,9 +701,14 @@ editing_stopped (GnomeIconTextItem *iti, gpointer data)
 	setup_icon_dnd_actions (dii);
 }
 
-/* Used to open a desktop icon when the user double-clicks on it */
+/**
+ * desktop_icon_info_open:
+ * @dii: The desktop icon to open.
+ * 
+ * Opens the specified desktop icon when the user double-clicks on it.
+ **/
 void
-desktop_icon_open (struct desktop_icon_info *dii)
+desktop_icon_info_open (DesktopIconInfo *dii)
 {
 	char *filename;
 	file_entry *fe;
@@ -739,7 +726,7 @@ desktop_icon_open (struct desktop_icon_info *dii)
 }
 
 void
-desktop_icon_delete (struct desktop_icon_info *dii)
+desktop_icon_info_delete (DesktopIconInfo *dii)
 {
 	char *full_name;
 	struct stat s;
@@ -756,23 +743,25 @@ desktop_icon_delete (struct desktop_icon_info *dii)
 		erase_dir (full_name, &progress_count, &progress_bytes);
 	else
 		erase_file (full_name, &progress_count, &progress_bytes, TRUE);
+
+	gmeta_del_icon_pos (full_name);
 	g_free (full_name);
 	destroy_op_win ();
 
 	/* 2. Destroy the dicon */
-	desktop_icon_destroy (dii);
+	desktop_icon_info_destroy (dii);
 }
 
 /* Used to execute the popup menu for desktop icons */
 static void
-do_popup_menu (struct desktop_icon_info *dii, GdkEventButton *event)
+do_popup_menu (DesktopIconInfo *dii, GdkEventButton *event)
 {
 	char *filename;
 
 	filename = g_concat_dir_and_file (desktop_directory, dii->filename);
 
 	if (gpopup_do_popup (event, NULL, dii, 0, filename) != -1)
-		reload_desktop_icons (TRUE, 0, 0); /* bleah */
+		reload_desktop_icons (0, 0);
 
 	g_free (filename);
 }
@@ -783,7 +772,7 @@ do_popup_menu (struct desktop_icon_info *dii, GdkEventButton *event)
 static gint
 window_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 	GtkWidget *parent;
 
 	dii = data;
@@ -815,7 +804,7 @@ window_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
 static gint
 icon_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 	int retval;
 
 	dii = data;
@@ -858,7 +847,7 @@ icon_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
 		if (event->button != 1)
 			break;
 
-		desktop_icon_open (dii);
+		desktop_icon_info_open (dii);
 		retval = TRUE;
 		break;
 
@@ -880,7 +869,7 @@ icon_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
 static gint
 icon_button_release (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 
 	dii = data;
 
@@ -902,7 +891,7 @@ icon_button_release (GtkWidget *widget, GdkEventButton *event, gpointer data)
 static gint
 icon_button_release_after (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 
 	dii = data;
 
@@ -935,7 +924,7 @@ icon_button_release_after (GtkWidget *widget, GdkEventButton *event, gpointer da
 static void
 drag_begin (GtkWidget *widget, GdkDragContext *context, gpointer data)
 {
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 	DesktopIcon *dicon;
 	GtkArg args[3];
 	GdkImlibImage *im;
@@ -986,7 +975,7 @@ build_selected_icons_uri_list (int *len)
 {
 	int i;
 	GList *l;
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 	char *filelist, *p;
 	int desktop_dir_len;
 
@@ -1041,7 +1030,7 @@ static void
 drag_data_get (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data,
 	       guint info, guint32 time, gpointer data)
 {
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 	char *filelist;
 	int len;
 	GList *files;
@@ -1081,7 +1070,7 @@ drag_data_get (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *sel
 
 /* Set up a desktop icon as a DnD source */
 static void
-setup_icon_dnd_source (struct desktop_icon_info *dii)
+setup_icon_dnd_source (DesktopIconInfo *dii)
 {
 	setup_icon_dnd_actions (dii);
 
@@ -1097,9 +1086,7 @@ setup_icon_dnd_source (struct desktop_icon_info *dii)
  * drop_on_file_entry
  */
 static void
-desktop_icon_drop_uri_list (struct desktop_icon_info *dii,
-			    GdkDragContext           *context,
-			    GtkSelectionData         *data)
+desktop_icon_drop_uri_list (DesktopIconInfo *dii, GdkDragContext *context, GtkSelectionData *data)
 {
 	char *filename;
 	file_entry *fe;
@@ -1159,7 +1146,7 @@ static void
 icon_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 			 GtkSelectionData *data, guint info, guint time, gpointer user_data)
 {
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 
 	dii = user_data;
 
@@ -1175,7 +1162,7 @@ icon_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gin
 
 /* Set up a desktop icon as a DnD destination */
 static void
-setup_icon_dnd_dest (struct desktop_icon_info *dii)
+setup_icon_dnd_dest (DesktopIconInfo *dii)
 {
 	char *filename;
 	file_entry *fe;
@@ -1222,10 +1209,10 @@ setup_icon_dnd_dest (struct desktop_icon_info *dii)
  * positioning trying to start at the specified coordinates.  It does
  * not show the icon.
  */
-static struct desktop_icon_info *
+static DesktopIconInfo *
 desktop_icon_info_new (char *filename, int auto_pos, int xpos, int ypos)
 {
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 	file_entry *fe;
 	char *full_name;
 	GdkImlibImage *icon_im;
@@ -1236,7 +1223,7 @@ desktop_icon_info_new (char *filename, int auto_pos, int xpos, int ypos)
 	fe = file_entry_from_file (full_name);
 	icon_im = gicon_get_icon_for_file_speed (fe, FALSE);
 
-	dii = g_new (struct desktop_icon_info, 1);
+	dii = g_new (DesktopIconInfo, 1);
 	dii->dicon = desktop_icon_new (icon_im, filename);
 	dii->x = 0;
 	dii->y = 0;
@@ -1285,12 +1272,14 @@ desktop_icon_info_new (char *filename, int auto_pos, int xpos, int ypos)
 	return dii;
 }
 
-/*
- * Frees a desktop icon information structure, and destroy the icon
- * widget.  Does not remove the structure from the desktop_icons list!
- */
+/**
+ * desktop_icon_info_destroy:
+ * @dii: The desktop icon to destroy
+ *
+ * Destroys the specified desktop icon.
+ **/
 void
-desktop_icon_destroy (struct desktop_icon_info *dii)
+desktop_icon_info_destroy (DesktopIconInfo *dii)
 {
 	gtk_widget_destroy (dii->dicon);
 	remove_from_slot (dii);
@@ -1450,13 +1439,13 @@ setup_xdnd_proxy (guint32 xid, GdkWindow *proxy_window)
 }
 
 /* Returns the desktop icon that started the drag from the specified context */
-struct desktop_icon_info *
+static DesktopIconInfo *
 find_icon_by_drag_context (GdkDragContext *context)
 {
 	GtkWidget *source;
 	int i;
 	GList *l;
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 
 	source = gtk_drag_get_source_widget (context);
 	if (!source)
@@ -1482,7 +1471,7 @@ find_icon_by_drag_context (GdkDragContext *context)
 static void
 drop_desktop_icons (GdkDragContext *context, GtkSelectionData *data, int x, int y)
 {
-	struct desktop_icon_info *source_dii, *dii;
+	DesktopIconInfo *source_dii, *dii;
 	int dx, dy;
 	int i;
 	GList *l;
@@ -1553,7 +1542,7 @@ desktop_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x,
 	case TARGET_URI_LIST:
 		retval = gdnd_drop_on_directory (context, data, desktop_directory);
 		if (retval)
-			reload_desktop_icons (TRUE, x, y);
+			reload_desktop_icons (x, y);
 
 		break;
 
@@ -1698,7 +1687,7 @@ store_temp_selection (void)
 {
 	int i;
 	GList *l;
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 
 	for (i = 0; i < (layout_cols * layout_rows); i++)
 		for (l = layout_slots[i].icons; l; l = l->next) {
@@ -1716,7 +1705,7 @@ store_temp_selection (void)
  * area, or FALSE otherwise.
  */
 static int
-icon_is_in_area (struct desktop_icon_info *dii, int x1, int y1, int x2, int y2)
+icon_is_in_area (DesktopIconInfo *dii, int x1, int y1, int x2, int y2)
 {
 	DesktopIcon *dicon;
 
@@ -1756,7 +1745,7 @@ update_drag_selection (int x, int y)
 	int x1, y1, x2, y2;
 	int i;
 	GList *l;
-	struct desktop_icon_info *dii;
+	DesktopIconInfo *dii;
 	int additive, invert, in_area;
 
 	if (click_start_x < x) {
@@ -2073,7 +2062,7 @@ desktop_init (void)
 {
 	create_layout_info ();
 	create_desktop_dir ();
-	load_desktop_icons (FALSE, 0, 0);
+	reload_desktop_icons (0, 0);
 	setup_desktop_dnd ();
 	setup_desktop_clicks ();
 }
