@@ -41,11 +41,17 @@ gmeta_get_icon_pos (char *filename, int *x, int *y)
 	if (gnome_metadata_get (filename, ICON_POSITION, &size, &buf) != 0)
 		return FALSE;
 
-	if (!buf || (sscanf (buf, "%d%d", &tx, &ty) != 2)) {
+	if (!buf){
 		g_warning ("Invalid metadata for \"%s\"'s icon position, using auto-placement", filename);
 		return FALSE;
 	}
 
+
+	if ((sscanf (buf, "%d%d", &tx, &ty) != 2)) {
+		g_free (buf);
+		return FALSE;
+	}
+	g_free (buf);
 	*x = tx;
 	*y = ty;
 	return TRUE;
@@ -72,3 +78,18 @@ gmeta_set_icon_pos (char *filename, int x, int y)
 	if (gnome_metadata_set (filename, ICON_POSITION, strlen (buf) + 1, buf) != 0)
 		g_warning ("Error setting the icon position metadata for \"%s\"", filename);
 }
+
+/**
+ * gmeta_del_icon_pos:
+ * @filename: The filename for which to delete icon position information.
+ * 
+ * Deletes the icon position information for the specified file.
+ **/
+void
+gmeta_del_icon_pos (char *filename)
+{
+	g_return_if_fail (filename != NULL);
+
+	gnome_metadata_remove (filename, ICON_POSITION);
+}
+
