@@ -44,6 +44,9 @@
 #include "dev.xpm"
 #include "listing-list.xpm"
 #include "listing-iconic.xpm"
+#include "listing-custom.xpm"
+#include "listing-brief-list.xpm"
+
 
 /* This is used to initialize our pixmaps */
 static int pixmaps_ready;
@@ -53,6 +56,7 @@ GdkPixmap *icon_link_pixmap;
 GdkBitmap *icon_link_mask;
 GdkPixmap *icon_dev_pixmap;
 GdkBitmap *icon_dev_mask;
+
 static GtkTargetEntry drag_types [] = {
 	{ "text/uri-list", 0, TARGET_URI_LIST },
 	{ "text/plain",    0, TARGET_TEXT_PLAIN },
@@ -162,7 +166,7 @@ panel_fill_panel_list (WPanel *panel)
 	int i, col, type_col, color;
 	char  **texts;
 
-	texts = malloc (sizeof (char *) * items);
+	texts = malloc (sizeof (char *) * (items+1));
 
 	gtk_clist_freeze (GTK_CLIST (cl));
 	gtk_clist_clear (GTK_CLIST (cl));
@@ -170,6 +174,9 @@ panel_fill_panel_list (WPanel *panel)
 	/* which column holds the type information */
 	type_col = -1;
 
+	g_assert (items == cl->columns);
+		  
+	texts [items] = NULL;
 	for (i = 0; i < top; i++){
 		file_entry *fe = &panel->dir.list [i];
 		format_e *format = panel->format;
@@ -1018,6 +1025,85 @@ panel_clist_drag_leave (GtkWidget *widget, GdkDragContext *ctx, guint time, void
 	}
 }
 
+/**
+ * panel_icon_list_scrolling_is_desirable:
+ *
+ * If the cursor is in a position close to either edge (top or bottom)
+ * and there is possible to scroll the window, this routine returns
+ * true.
+ */
+static gboolean
+panel_icon_list_scrolling_is_desirable (WPanel *panel, int x, int y)
+{
+	GtkAdjustment *va;
+
+	va = GNOME_ICON_LIST (panel->icons)->adj;
+
+	if (y < 10){
+		if (va->value > va->lower)
+			return TRUE;
+	} else {
+		if (y > (GTK_WIDGET (panel->icons)->allocation.height-20)){
+			if (va->value < va->upper)
+				return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+
+/**
+ * panel_icon_list_scroll:
+ *
+ * Timer callback invoked to scroll the clist window
+ */
+static gboolean
+panel_icon_list_scroll (gpointer data)
+{
+	WPanel *panel = data;
+	GtkAdjustment *va;
+
+	va = GNOME_ICON_LIST (panel->icons)->adj;
+
+	if (panel->drag_motion_y < 10)
+		gtk_adjustment_set_value (va, va->value - va->step_increment);
+	else{
+		gtk_adjustment_set_value (va, va->value + va->step_increment);
+	}
+	return TRUE;
+}
+/**
+ * panel_icon_list_drag_motion:
+ *
+ * Invoked when an application dragging over us has the the cursor moved.
+ * If we are close to the top or bottom, we scroll the window
+ */
+static gboolean
+panel_icon_list_drag_motion (GtkWidget *widget, GdkDragContext *ctx, int x, int y, guint time, void *data)
+{
+	WPanel *panel = data;
+	
+	panel_setup_drag_motion (panel, x, y, panel_icon_list_scrolling_is_desirable, panel_icon_list_scroll);
+	return TRUE;
+}
+
+/**
+ * panel_icon_list_drag_leave:
+ *
+ * Invoked when the dragged object has left our region
+ */
+static void
+panel_icon_list_drag_leave (GtkWidget *widget, GdkDragContext *ctx, guint time, void *data)
+{
+	WPanel *panel = data;
+
+	if (panel->timer_id != -1){
+		gtk_timeout_remove (panel->timer_id);
+		panel->timer_id = -1;
+	}
+}
+
 /*
  * Create, setup the file listing display.
  */
@@ -1062,6 +1148,10 @@ panel_create_file_list (WPanel *panel)
 			   drop_types, ELEMENTS (drop_types),
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 
+	gtk_drag_source_set (GTK_WIDGET (file_list), GDK_BUTTON1_MASK,
+			     drag_types, ELEMENTS (drag_types),
+			     GDK_ACTION_LINK | GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_ASK);
+
 	gtk_signal_connect (GTK_OBJECT (file_list), "drag_data_get",
 			    GTK_SIGNAL_FUNC (panel_drag_data_get), panel);
 	gtk_signal_connect (GTK_OBJECT (file_list), "drag_data_delete",
@@ -1254,6 +1344,10 @@ panel_create_icon_display (WPanel *panel)
 			   drop_types, ELEMENTS (drop_types),
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 
+	gtk_drag_source_set (GTK_WIDGET (ilist), GDK_BUTTON1_MASK,
+			     drag_types, ELEMENTS (drag_types),
+			     GDK_ACTION_LINK | GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_ASK);
+
 	gtk_signal_connect (GTK_OBJECT (ilist), "drag_data_get",
 			    GTK_SIGNAL_FUNC (panel_drag_data_get),
 			    panel);
@@ -1280,6 +1374,15 @@ panel_create_icon_display (WPanel *panel)
 			    GTK_SIGNAL_FUNC (panel_widget_motion),
 			    panel);
 
+	/*
+	 * This signal is provide for scrolling the main window if
+	 * data is being dragged
+	 */
+	gtk_signal_connect (GTK_OBJECT (ilist), "drag_motion",
+			    GTK_SIGNAL_FUNC (panel_icon_list_drag_motion), panel);
+	gtk_signal_connect (GTK_OBJECT (ilist), "drag_leave",
+			    GTK_SIGNAL_FUNC (panel_icon_list_drag_leave), panel);
+	
 	return GTK_WIDGET (ilist);
 }
 
@@ -1293,12 +1396,8 @@ panel_switch_new_display_mode (WPanel *panel)
 
 	panel->list = panel_create_file_list (panel);
 	gtk_widget_destroy (old_list);
-	gtk_table_attach (GTK_TABLE (panel->view_table), panel->list, 0, 1, 0, 1,
-			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
-			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
-			  0, 0);
-	gtk_widget_show (panel->list);
-	panel_update_contents (panel);
+	gtk_container_add (GTK_CONTAINER (panel->panel_listbox), panel->list);
+	gtk_widget_show_all (panel->list);
 }
 
 static GtkWidget *
@@ -1314,9 +1413,9 @@ panel_create_cwd (Dlg_head *h, WPanel *panel, void **entry)
 
 	*entry = in;
 	/* FIXME: for now, we set the usize.  Ultimately, toolbar
-	 * will let you expand it.
+	 * will let you expand it, we hope.
 	 */
-	gtk_widget_set_usize (GTK_WIDGET (in->widget.wdata), 320, -1);
+	gtk_widget_set_usize (GTK_WIDGET (in->widget.wdata), 296, -1);
 	return GTK_WIDGET (in->widget.wdata);
 }
 
@@ -1967,19 +2066,37 @@ do_switch_to_iconic (GtkWidget *widget, WPanel *panel)
 		return;
 	panel->list_type = list_icons;
 	set_panel_formats (panel);
-	paint_panel (panel);
-	do_refresh ();
+	panel_update_contents (panel);
 }
 
 static void
-do_switch_to_listing (GtkWidget *widget, WPanel *panel)
+do_switch_to_brief_listing (GtkWidget *widget, WPanel *panel)
 {
-	if (panel->list_type != list_icons)
+	if (panel->list_type == list_brief)
+		return;
+	panel->list_type = list_brief;
+	set_panel_formats (panel);
+	panel_update_contents (panel);
+}
+
+static void
+do_switch_to_full_listing (GtkWidget *widget, WPanel *panel)
+{
+	if (panel->list_type == list_full)
 		return;
 	panel->list_type = list_full;
 	set_panel_formats (panel);
-	paint_panel (panel);
-	do_refresh ();
+	panel_update_contents (panel);
+}
+
+static void
+do_switch_to_custom_listing (GtkWidget *widget, WPanel *panel)
+{
+	if (panel->list_type == list_user)
+		return;
+	panel->list_type = list_user;
+	set_panel_formats (panel);
+	panel_update_contents (panel);
 }
 
 static GtkWidget *
@@ -1989,9 +2106,19 @@ button_switch_to_icon (WPanel *panel)
 }
 
 static GtkWidget *
-button_switch_to_listing (WPanel *panel)
+button_switch_to_full_listing (WPanel *panel)
 {
-	return button_switch_to (listing_list_xpm, GTK_SIGNAL_FUNC (do_switch_to_listing), panel);
+	return button_switch_to (listing_list_xpm, GTK_SIGNAL_FUNC (do_switch_to_full_listing), panel);
+}
+static GtkWidget *
+button_switch_to_brief_listing (WPanel *panel)
+{
+	return button_switch_to (listing_brief_list_xpm, GTK_SIGNAL_FUNC (do_switch_to_brief_listing), panel);
+}
+static GtkWidget *
+button_switch_to_custom_listing (WPanel *panel)
+{
+	return button_switch_to (listing_custom_xpm, GTK_SIGNAL_FUNC (do_switch_to_custom_listing), panel);
 }
 
 void
@@ -2026,6 +2153,7 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 	gtk_widget_show (panel->notebook);
 	
 	panel->icons = panel_create_icon_display (panel);
+	gtk_widget_show (panel->icons);
 	panel->scrollbar = gtk_vscrollbar_new (GNOME_ICON_LIST (panel->icons)->adj);
 	gtk_widget_show (panel->scrollbar);
 
@@ -2036,9 +2164,15 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 	box = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (box), panel->icons, TRUE, TRUE, 0);
 	gtk_box_pack_end (GTK_BOX (box), panel->scrollbar, FALSE, TRUE, 0);
+	gtk_widget_show (box);
+	
+	panel->panel_listbox = gtk_event_box_new ();
+	gtk_widget_show (panel->panel_listbox);
+	gtk_container_add (GTK_CONTAINER (panel->panel_listbox), panel->list);
 
+	
 	gtk_notebook_append_page (GTK_NOTEBOOK (panel->notebook), box, NULL);
-	gtk_notebook_append_page (GTK_NOTEBOOK (panel->notebook), panel->list, NULL);
+	gtk_notebook_append_page (GTK_NOTEBOOK (panel->notebook), panel->panel_listbox, NULL);
 	gtk_notebook_set_page (GTK_NOTEBOOK (panel->notebook), panel->list_type == list_icons ? 0 : 1);
 	gtk_widget_show_all (box);
 	gtk_widget_show (panel->list);
@@ -2107,10 +2241,16 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 	gtk_toolbar_append_space (GTK_TOOLBAR (status_line));
 	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
 				   button_switch_to_icon (panel),
-				   "Switch view to icon view.", NULL);
+				   "Icon view.", NULL);
 	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
-				   button_switch_to_listing (panel),
-				   "Switch view to detailed view.", NULL);
+				   button_switch_to_brief_listing (panel),
+				   "Brief view.", NULL);
+	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
+				   button_switch_to_full_listing (panel),
+				   "Detailed view.", NULL);
+	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
+				   button_switch_to_custom_listing (panel),
+				   "Custom view.", NULL);
 	dock =  gnome_dock_item_new ("gmc-toolbar", GNOME_DOCK_ITEM_BEH_EXCLUSIVE | GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL);
 	gtk_container_add (GTK_CONTAINER(dock),status_line);
 	gnome_dock_add_item (GNOME_DOCK(GNOME_APP (panel->xwindow)->dock),
@@ -2270,12 +2410,18 @@ paint_frame (WPanel *panel)
 void
 x_reset_sort_labels (WPanel *panel)
 {
+	int page;
+		
 	if (!panel->notebook)
 		return;
 
-	gtk_notebook_set_page (
-		GTK_NOTEBOOK (panel->notebook),
-		panel->list_type == list_icons ? 0 : 1);
+	if (panel->list_type == list_icons){
+		page = 0;
+	} else {
+		page = 1;
+		panel_switch_new_display_mode (panel);
+	}
+	gtk_notebook_set_page (GTK_NOTEBOOK (panel->notebook), page);
 }
 
 /* Releases all of the X resources allocated */
