@@ -25,7 +25,7 @@
 #include "color.h"
 #include "x.h"
 
-/* "$Id: color.c,v 1.6 1998/05/17 03:27:53 pavel Exp $" */
+/* "$Id: color.c,v 1.11 1998/12/29 16:52:56 norbert Exp $" */
 
 /* To avoid excessive calls to ncurses' has_colors () */
 int   hascolors = 0;
@@ -67,7 +67,7 @@ struct colorpair {
 #endif
 
 struct colorpair color_map [] = {
-    { "normal=",     0, 0 },	/* normal */
+    { "normal=",     0, 0 },	/* normal */               /*  1 */
     { "selected=",   0, 0 },	/* selected */
     { "marked=",     0, 0 },	/* marked */
     { "markselect=", 0, 0 },	/* marked/selected */
@@ -76,32 +76,43 @@ struct colorpair color_map [] = {
     { "reverse=",    0, 0 },	/* reverse */
 
     /* Dialog colors */
-    { "dnormal=",    0, 0 },	/* Dialog normal */
+    { "dnormal=",    0, 0 },	/* Dialog normal */        /*  8 */
     { "dfocus=",     0, 0 },	/* Dialog focused */
     { "dhotnormal=", 0, 0 },	/* Dialog normal/hot */
     { "dhotfocus=",  0, 0 },	/* Dialog focused/hot */
     
-    { "viewunderline=", 0, 0 },	/* _\b? sequence in view */
-    { "menusel=",    0, 0 },	/* Menu selected color */
+    { "viewunderline=", 0, 0 },	/* _\b? sequence in view, underline in editor */
+    { "menusel=",    0, 0 },	/* Menu selected color */  /* 13 */
     { "menuhot=",    0, 0 },    /* Color for menu hotkeys */
     { "menuhotsel=", 0, 0 },    /* Menu hotkeys/selected entry */
     
-    { "helpnormal=", 0, 0 },    /* Help normal */
+    { "helpnormal=", 0, 0 },    /* Help normal */          /* 16 */
     { "helpitalic=", 0, 0 },    /* Italic in help */
     { "helpbold=",   0, 0 },    /* Bold in help */
     { "helplink=",   0, 0 },    /* Not selected hyperlink */
     { "helpslink=",  0, 0 },    /* Selected hyperlink */
     
-    { "gauge=",      0, 0 },    /* Color of the progress bar (percentage) */
+    { "gauge=",      0, 0 },    /* Color of the progress bar (percentage) *//* 21 */
     { "input=",      0, 0 },
  
     /* Per file types colors */
-    { "directory=",  0, 0 },
-    { "execute=",    0, 0 },
-    { "link=",       0, 0 },
+    { "directory=",  0, 0 },                               /*  23 */
+    { "executable=", 0, 0 },
+    { "link=",       0, 0 },  /* symbolic link (neither stalled nor link to directory) */
+    { "stalledlink=",0, 0 },  /* stalled symbolic link */
     { "device=",     0, 0 },
-    { "special=",    0, 0 },
-    { "core=",       0, 0 },
+    { "special=",    0, 0 }, /* sockets, fifo */
+    { "core=",       0, 0 }, /* core files */              /* 29 */
+
+    { 0,             0, 0 }, /* not usable (DEFAULT_COLOR_INDEX) *//* 30 */
+    { 0,             0, 0 }, /* unused */
+    { 0,             0, 0 }, /* not usable (A_REVERSE) */
+    { 0,             0, 0 }, /* not usable (A_REVERSE_BOLD) */
+
+/* editor colors start at 34 */
+    { "editnormal=",     0, 0 },	/* normal */       /* 34 */
+    { "editbold=",       0, 0 },	/* search->found */
+    { "editmarked=",     0, 0 },	/* marked/selected */
 };
 
 struct color_table_s {
@@ -126,7 +137,8 @@ struct color_table_s color_table [] = {
     { "cyan",          COLOR_CYAN    },
     { "brightcyan",    COLOR_CYAN    | A_BOLD },
     { "lightgray",     COLOR_WHITE },
-    { "white",         COLOR_WHITE   | A_BOLD }
+    { "white",         COLOR_WHITE   | A_BOLD },
+    { "default",       0 } /* hack for transparent background */
 };
 
 #ifdef HAVE_GNOME
@@ -195,7 +207,11 @@ void configure_colors_string (char *the_color_string)
 
 	found = 0;
 	for (i = 0; i < ELEMENTS(color_map); i++){
-	    int klen = strlen (color_map [i].name);
+	    int klen;
+
+            if (!color_map [i].name)
+                continue;
+            klen = strlen (color_map [i].name);
 
 	    if (strncmp (color_string, color_map [i].name, klen) == 0){
 		color_string += klen;
@@ -216,13 +232,9 @@ void configure_colors_string (char *the_color_string)
 static void configure_colors (void)
 {
     extern char *command_line_colors;
-#ifndef HAVE_TK
-#ifndef HAVE_XVIEW
     extern char *default_edition_colors;
     
     configure_colors_string (default_edition_colors);
-#endif
-#endif
     configure_colors_string (setup_color_string);
     configure_colors_string (term_color_string);
     configure_colors_string (getenv ("MC_COLOR_TABLE"));
@@ -230,7 +242,7 @@ static void configure_colors (void)
 }
 
 #ifndef HAVE_SLANG
-#define MAX_PAIRS 30
+#define MAX_PAIRS 34
 int attr_pairs [MAX_PAIRS];
 #endif
 
@@ -250,10 +262,10 @@ init_colors (void)
 	int i;
 	
 	use_colors = 1;
-	start_color ();
 	configure_colors ();
-	for (i = 0; i < ELEMENTS (color_map); i++)
-	    init_pair (i+1, color_map_fg(i), color_map_bg(i));
+	for (i = 0; i < ELEMENTS (color_map); i++) 
+            if (color_map [i].name)
+                init_pair (i+1, color_map_fg(i), color_map_bg(i));
 	load_dialog_colors ();
 }
 #else
@@ -268,7 +280,9 @@ void init_colors (void)
     }
 
     if (use_colors){
+#ifndef HAVE_X
 	start_color ();
+#endif
 	configure_colors ();
 
 #ifndef HAVE_SLANG
@@ -281,19 +295,22 @@ void init_colors (void)
 #endif
 
 #if defined HAVE_SLANG && !defined(OS2_NT)
-	if (use_colors) { /* Hack to make COLOR_PAIR(31) be the default fg/bg
-	                     of the terminal */
+	if (use_colors) { /* Hack to make COLOR_PAIR(DEFAULT_COLOR_INDEX) 
+                             be the default fg/bg of the terminal */
 	    char *Norm_Vid = SLtt_tgetstr ("me");
 	    
 	    if (Norm_Vid == NULL)
 	        Norm_Vid = SLtt_tgetstr ("se");
 	    if (Norm_Vid == NULL)
 	        Norm_Vid = "\033[0m";
-	    SLtt_set_color_esc (31, Norm_Vid);
+	    SLtt_set_color_esc (DEFAULT_COLOR_INDEX, Norm_Vid);
 	}
 #endif	
 
 	for (i = 0; i < ELEMENTS (color_map); i++){
+            if (!color_map [i].name)
+                continue;
+
 	    init_pair (i+1, color_map_fg(i), color_map_bg(i));
 
 #ifndef HAVE_SLANG
