@@ -52,8 +52,7 @@
 #ifdef SCO_FLAVOR
 #include <sys/wait.h>
 #endif /* SCO_FLAVOR */
-
-/* "$Id: ext.c,v 1.7 1998/05/19 18:37:07 unammx Exp $" */
+#include <glib.h>
 
 /* If set, we execute the file command to check the file type */
 int use_file_to_check_type = 1;
@@ -102,7 +101,7 @@ quote_block (quote_func_t quote_func, char **quoting_block)
 	return result;
 }
 	     
-static void
+void
 exec_extension (char *filename, char *data, char **drops, int *move_dir, int start_line)
 {
     char *file_name;
@@ -124,6 +123,9 @@ exec_extension (char *filename, char *data, char **drops, int *move_dir, int sta
     struct stat mystat;
     quote_func_t quote_func = name_quote;
 
+    g_return_if_fail (filename != NULL);
+    g_return_if_fail (data != NULL);
+
     /* Avoid making a local copy if we are doing a cd */
     if (!vfs_file_is_local(filename))
 	do_local_copy = 1;
