@@ -65,7 +65,7 @@
 #define HISTORY_SIZE 20
 #define HELP_WINDOW_WIDTH 62
 
-/* "$Id: help.c,v 1.5 1998/09/22 13:06:59 proskin Exp $" */
+/* "$Id: help.c,v 1.7 1998/12/02 23:44:25 unammx Exp $" */
 
 static char *data;		/* Pointer to the loaded data file */
 static int help_lines = 18;	/* Lines in help viewer */
@@ -129,7 +129,8 @@ static int acs2pc (int acscode)
 
 /* returns the position where text was found in the start buffer */
 /* or 0 if not found */
-char *search_string (char *start, char *text)
+static char *
+search_string (char *start, char *text)
 {
     char *d = text;
     char *e = start;
@@ -549,19 +550,19 @@ static int help_event (Gpm_Event *event, Widget *w)
 }
 
 /* show help */
-void help_help_cmd (Dlg_head *h)
+static void
+help_help_cmd (Dlg_head *h)
 {
     history_ptr = (history_ptr+1) % HISTORY_SIZE;
     history [history_ptr].page = currentpoint;
     history [history_ptr].link = selected_item;
     currentpoint = startpoint = search_string (data, "[How to use help]") + 1;
     selected_item = NULL;
-#ifndef HAVE_XVIEW    
     help_callback (h, 0, DLG_DRAW);
-#endif    
 }
 
-void help_index_cmd (Dlg_head *h)
+static void
+help_index_cmd (Dlg_head *h)
 {
     char *new_item;
 
@@ -575,9 +576,7 @@ void help_index_cmd (Dlg_head *h)
     else
 	currentpoint = startpoint = new_item + 1;
     selected_item = NULL;
-#ifndef HAVE_XVIEW    
     help_callback (h, 0, DLG_DRAW);
-#endif    
 }
 
 static void quit_cmd (void *x)
@@ -593,9 +592,7 @@ static void prev_node_cmd (Dlg_head *h)
     if (history_ptr < 0)
 	history_ptr = HISTORY_SIZE-1;
     
-#ifndef HAVE_XVIEW    
     help_callback (h, 0, DLG_DRAW);
-#endif    
 }
 
 static int md_callback (Dlg_head *h, Widget *w, int msg, int par)
@@ -755,13 +752,15 @@ static int help_callback (struct Dlg_head *h, int id, int msg)
     return 0;
 }
 
-void interactive_display_finish (void)
+static void
+interactive_display_finish (void)
 {
     clear_link_areas ();
     free (data);
 }
 
-void interactive_display (char *filename, char *node)
+void
+interactive_display (char *filename, char *node)
 {
     WButtonBar *help_bar;
     Widget     *md;
