@@ -139,7 +139,7 @@
 #include <glib.h>
 #include "popt.h"
 
-/* "$Id: main.c,v 1.63 1998/12/30 02:51:13 unammx Exp $" */
+/* "$Id: main.c,v 1.68 1999/01/03 18:34:48 unammx Exp $" */
 
 /* When the modes are active, left_panel, right_panel and tree_panel */
 /* Point to a proper data structure.  You should check with the functions */
@@ -446,6 +446,11 @@ update_one_panel_widget (WPanel *panel, int force_update, char *current_file)
 
     if (free_pointer)
 	free (current_file);
+
+#ifdef HAVE_X
+    paint_panel (panel);
+    panel->dirty = 0;
+#endif
 }
 
 #ifndef PORT_HAS_UPDATE_PANELS
@@ -1153,21 +1158,8 @@ sort_cmd (void)
     p = MENU_PANEL;
     sort_order = sort_box (p->sort_type, &p->reverse, &p->case_sensitive);
 
-    if (sort_order == 0)
-	return;
+    panel_set_sort_order (p, sort_order);
 
-    p->sort_type = sort_order;
-
-    /* The directory is already sorted, we have to load the unsorted stuff */
-    if (sort_order == (sortfn *) unsorted){
-	char *current_file;
-	
-	current_file = strdup (cpanel->dir.list [cpanel->selected].fname);
-	panel_reload (cpanel);
-	try_to_select (cpanel, current_file);
-	free (current_file);
-    }
-    do_re_sort (p);
 }
 
 #ifndef HAVE_GNOME
@@ -2665,6 +2657,14 @@ static struct poptOption argument_table [] = {
     { NULL, 		0,			0, NULL, 0 }
 };
 
+void
+main_corba_register_server (void)
+{
+#ifdef HAVE_CORBA
+	corba_register_server ();
+#endif
+}
+
 static void
 handle_args (int argc, char *argv [])
 {
@@ -2681,13 +2681,12 @@ handle_args (int argc, char *argv [])
     orb = gnome_CORBA_init_with_popt_table (
 	    "gmc", VERSION, &argc, argv, argument_table, 0, &ctx, GNORBA_INIT_SERVER_FUNC, &ev);
 
-    corba_register_server ();
 #else
     gnome_init_with_popt_table ("gmc", VERSION, argc, argv, argument_table, 0, &ctx);
+#endif
     gtk_widget_push_visual (gdk_imlib_get_visual ());
     gtk_widget_push_colormap (gdk_imlib_get_colormap ());
 	
-#endif
     poptResetContext (ctx);
 #else
     ctx = poptGetContext ("mc", argc, argv, argument_table, 0);
@@ -2811,16 +2810,12 @@ compatibility_move_mc_files (void)
 
 int main (int argc, char *argv [])
 {
-#ifndef OS2_NT
-    /* Backward compatibility: Gives up privileges in case someone
-       installed the mc as setuid */
-    setuid (getuid ());
-#endif
     /* We had LC_CTYPE before, LC_ALL includs LC_TYPE as well */
     setlocale (LC_ALL, "");
     bindtextdomain ("mc", LOCALEDIR);
     textdomain ("mc");
 
+    /* This is here to debug the CORBA async invocations */
     {
 	    volatile  int i = 1;
 
@@ -2833,6 +2828,13 @@ int main (int argc, char *argv [])
     
     OS_Setup ();
 
+    /* This variable is used by the subshell */
+    home_dir = getenv ("HOME");
+    if (!home_dir) {
+	    /* mc_home was computed by OS_Setup */
+	    home_dir = mc_home; 
+    }
+
     vfs_init ();
 
 #ifdef HAVE_X
@@ -2917,16 +2919,6 @@ int main (int argc, char *argv [])
     /* Install the SIGCHLD handler; must be done before init_subshell() */
     init_sigchld ();
     
-    /* This variable is used by the subshell */
-    home_dir = getenv ("HOME");
-    if (!home_dir) {
-#ifndef OS2_NT
-    home_dir = PATH_SEP_STR;
-#else
-    home_dir = mc_home; /* LIBDIR, calculated in OS_Setup() */
-#endif
-    }
-
     compatibility_move_mc_files ();
     
 #   ifdef HAVE_X
@@ -2973,9 +2965,6 @@ int main (int argc, char *argv [])
         application_keypad_mode ();
 #   endif
 
-    /* The directory hot list */
-    load_hotlist ();
-
     if (show_change_notice){
 	message (1, _(" Notice "),
 		 _(" The Midnight Commander configuration files \n"
