@@ -37,6 +37,7 @@
 #include "main.h"
 #include "user.h"
 #include "layout.h"
+#include "setup.h"
 #include "../vfs/vfs.h"
 
 /* For the simple listbox manager */
@@ -46,7 +47,7 @@
 
 #include "view.h" /* for default_* externs */
 
-/* "$Id: user.c,v 1.6 1998/05/08 02:25:27 unammx Exp $" */
+/* "$Id: user.c,v 1.8 1998/12/02 21:28:05 unammx Exp $" */
 
 #define MAX_ENTRIES 40
 #define MAX_ENTRY_LEN 60
@@ -171,7 +172,8 @@ int check_format_var (char *p, char **v)
 }
 
 /* strip file's extension */
-char *strip_ext(char *ss)
+static char *
+strip_ext(char *ss)
 {
     register char *s = ss;
     char *e = NULL;
@@ -376,7 +378,8 @@ static char *test_condition (char *p, int *condition)
 }
 
 /* General purpose condition debug output handler */
-void debug_out (char *start, char *end, int cond)
+static void
+debug_out (char *start, char *end, int cond)
 {
     static char msg [256];
     int len;
@@ -487,7 +490,8 @@ static char *test_line (char *p, int *result)
 }
 
 /* FIXME: recode this routine on version 3.0, it could be cleaner */
-void execute_menu_command (char *s)
+static void
+execute_menu_command (char *s)
 {
     char *commands;
     FILE *cmd_file;
@@ -601,7 +605,6 @@ menu_file_own(char* path)
 	}
 	else
 	{
-		extern int verbose;
 		if (verbose)
 		{
 			message (0, _(" Warning -- ignoring file "),
