@@ -84,7 +84,7 @@ int message (int error, char *header, char *text, ...);
 #include "user.h"		/* expand_format */
 #include "../vfs/vfs.h"
 
-/* "$Id: util.c,v 1.8 1998/09/22 11:20:32 pavel Exp $" */
+/* "$Id: util.c,v 1.9 1998/12/02 21:28:06 unammx Exp $" */
 
 char app_text [] = "Midnight-Commander";
 
@@ -861,7 +861,8 @@ char *get_current_wd (char *buffer, int size)
 
 #define CHECK(x) if (x == -1) return 0;
 
-long get_small_endian_long (int fd)
+static long
+get_small_endian_long (int fd)
 {
     unsigned char a, b, c, d;
 
