@@ -23,7 +23,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
  */
-/* "$Id: widget.c,v 1.22 1998/12/02 23:44:32 unammx Exp $" */
+/* "$Id: widget.c,v 1.23 1999/01/02 18:16:12 unammx Exp $" */
 
 #include <config.h>
 #include <string.h>
@@ -2208,7 +2208,8 @@ listbox_new (int y, int x, int width, int height,
     l->list   = l->top = l->current = 0;
     l->pos    = 0;
     l->width  = width;
-    l->height = height;
+    if (height <= 0)
+	    l->height = 0;
     l->count  = 0;
     l->top    = 0;
     l->current= 0;
