@@ -1,6 +1,14 @@
 #ifndef __VFS_H
 #define __VFS_H
 
+#include <config.h>
+#include <sys/stat.h>
+#include <dirent.h>
+
+#if 0
+#include "src/mad.h"
+#endif
+
 #if !defined(SCO_FLAVOR) || !defined(_SYS_SELECT_H)
 #	include <sys/time.h>	/* alex: this redefines struct timeval */
 #endif /* SCO_FLAVOR */
@@ -33,80 +41,95 @@ struct utimbuf {
     typedef void * vfsid;
 
     struct vfs_stamping;
-    
-    typedef struct vfs {
-        struct vfs *next;
+
+/*
+ * Notice: Andrej Borsenkow <borsenkow.msk@sni.de> reports system
+ * (RelianUNIX), where it is bad idea to define struct vfs. That system
+ * has include called <sys/vfs.h>, which contains things like vfs_t.
+ */
+
+    typedef struct _vfs vfs;
+
+    struct _vfs {
+        vfs   *next;
         char  *name;		/* "FIles over SHell" */
         int   flags;
 #define F_EXEC 1
 #define F_NET 2
-        char  *prefix;		/* "#fish:" */
+        char  *prefix;		/* "fish:" */
         void  *data;		/* this is for filesystem's own use */
         int   verrno;           /* can't use errno because glibc2 might define errno as function */
-        int   (*init)(struct vfs *me);	/* 1..initialized succesfully */
-        void  (*done)(struct vfs *me);
-        void  (*fill_names)(struct vfs *me, void (*)(char *));
-        int   (*which)(struct vfs *me, char *path); /* Returns '-1' if this path does not belong to this filesystem */
 
-	void  *(*open)(struct vfs *me, char *fname, int flags, int mode);
-	int   (*close)(void *vfs_info);
-	int   (*read)(void *vfs_info, char *buffer, int count);
-	int   (*write)(void *vfs_info, char *buf, int count);
-
-	void  *(*opendir)(struct vfs *me, char *dirname);
-	void  *(*readdir)(void *vfs_info);
-	int   (*closedir)(void *vfs_info);
- 	int   (*telldir)(void *vfs_info);
- 	void  (*seekdir)(void *vfs_info, int offset);
-
-	int  (*stat)(struct vfs *me, char *path, struct stat *buf);
-	int  (*lstat)(struct vfs *me, char *path, struct stat *buf);
-	int  (*fstat)(void *vfs_info, struct stat *buf);
-
-	int  (*chmod)(struct vfs *me, char *path, int mode);
-	int  (*chown)(struct vfs *me, char *path, int owner, int group);
-	int  (*utime)(struct vfs *me, char *path, struct utimbuf *times);
-
-	int  (*readlink)(struct vfs *me, char *path, char *buf, int size);
-	int  (*symlink)(struct vfs *me, char *n1, char *n2);
-	int  (*link)(struct vfs *me, char *p1, char *p2);
-	int  (*unlink)(struct vfs *me, char *path);
-	int  (*rename)(struct vfs *me, char *p1, char *p2);
-	int  (*chdir)(struct vfs *me, char *path);
-	int  (*ferrno)(struct vfs *me);
-	int  (*lseek)(void *vfs_info, off_t offset, int whence);
-	int  (*mknod)(struct vfs *me, char *path, int mode, int dev);
+        int   (*init)          (vfs *me);
+        void  (*done)          (vfs *me);
+        void  (*fill_names)    (vfs *me, void (*)(char *));
+			       
+        int   (*which)         (vfs *me, char *path);
+			       
+	void  *(*open) 	       (vfs *me, char *fname, int flags, int mode);
+	int   (*close) 	       (void *vfs_info);
+	int   (*read)  	       (void *vfs_info, char *buffer, int count);
+	int   (*write) 	       (void *vfs_info, char *buf, int count);
+			       
+	void  *(*opendir)      (vfs *me, char *dirname);
+	void  *(*readdir)      (void *vfs_info);
+	int   (*closedir)      (void *vfs_info);
+ 	int   (*telldir)       (void *vfs_info);
+ 	void  (*seekdir)       (void *vfs_info, int offset);
+			       
+	int  (*stat)           (vfs *me, char *path, struct stat *buf);
+	int  (*lstat)  	       (vfs *me, char *path, struct stat *buf);
+	int  (*fstat)  	       (void *vfs_info, struct stat *buf);
+			       
+	int  (*chmod)  	       (vfs *me, char *path, int mode);
+	int  (*chown)  	       (vfs *me, char *path, int owner, int group);
+	int  (*utime)  	       (vfs *me, char *path, struct utimbuf *times);
+			       
+	int  (*readlink)       (vfs *me, char *path, char *buf, int size);
+	int  (*symlink)        (vfs *me, char *n1, char *n2);
+	int  (*link)           (vfs *me, char *p1, char *p2);
+	int  (*unlink)         (vfs *me, char *path);
+	int  (*rename)         (vfs *me, char *p1, char *p2);
+	int  (*chdir)          (vfs *me, char *path);
+	int  (*ferrno)         (vfs *me);
+	int  (*lseek)          (void *vfs_info, off_t offset, int whence);
+	int  (*mknod)          (vfs *me, char *path, int mode, int dev);
 	
-	vfsid (*getid)(struct vfs *me, char *path, struct vfs_stamping **parent);
-	int  (*nothingisopen)(vfsid id);
-	void (*free)(vfsid id);
+	vfsid (*getid)         (vfs *me, char *path, struct vfs_stamping **
+				parent);
+	    
+	int  (*nothingisopen)  (vfsid id);
+	void (*free)           (vfsid id);
 	
-	char *(*getlocalcopy)(struct vfs *me, char *filename);
-	void (*ungetlocalcopy)(struct vfs *me, char *filename, char *local, int has_changed);
+	char *(*getlocalcopy)  (vfs *me, char *filename);
+	void (*ungetlocalcopy) (vfs *me, char *filename, char *local,
+				int has_changed);
 
-	int  (*mkdir)(struct vfs *me, char *path, mode_t mode);
-	int  (*rmdir)(struct vfs *me, char *path);
+	int  (*mkdir)          (vfs *me, char *path, mode_t mode);
+	int  (*rmdir)          (vfs *me, char *path);
 	
-	int  (*ctl)(void *vfs_info, int ctlop, int arg);
-	int  (*setctl)(struct vfs *me, char *path, int ctlop, char *arg);
+	int  (*ctl)            (void *vfs_info, int ctlop, int arg);
+	int  (*setctl)         (vfs *me, char *path, int ctlop, char *arg);
 #ifdef HAVE_MMAP
-	caddr_t (*mmap)(struct vfs *me, caddr_t addr, size_t len, int prot, int flags, void *vfs_info, off_t offset);
-	int (*munmap)(struct vfs *me, caddr_t addr, size_t len, void *vfs_info);
+	caddr_t (*mmap)        (vfs *me, caddr_t addr, size_t len, int prot,
+				int flags, void *vfs_info, off_t offset);
+	int (*munmap)          (vfs *me, caddr_t addr, size_t len,
+				void *vfs_info);
 #endif	
-    } vfs;
+    };
 
     /* Other file systems */
-    extern vfs local_vfs_ops;
-    extern vfs tarfs_vfs_ops;
+    extern vfs vfs_local_ops;
+    extern vfs vfs_tarfs_ops;
 
-    extern vfs ftpfs_vfs_ops;
-    extern vfs fish_vfs_ops;
-    extern vfs mcfs_vfs_ops;
+    extern vfs vfs_ftpfs_ops;
+    extern vfs vfs_fish_ops;
+    extern vfs vfs_mcfs_ops;
     
-    extern vfs extfs_vfs_ops;
-    extern vfs sfs_vfs_ops;
+    extern vfs vfs_extfs_ops;
+    extern vfs vfs_sfs_ops;
 
-    extern vfs undelfs_vfs_ops;
+    extern vfs vfs_undelfs_ops;
 
     struct vfs_stamping {
         vfs *v;
@@ -125,11 +148,14 @@ struct utimbuf {
     vfsid vfs_ncs_getid (vfs *nvfs, char *dir, struct vfs_stamping **par);
     void vfs_rm_parents (struct vfs_stamping *stamp);
     char *vfs_path (char *path);
+    char *vfs_strip_suffix_from_filename (char *filename);
     char *vfs_canon (char *path);
     char *mc_get_current_wd (char *buffer, int bufsize);
     int vfs_current_is_local (void);
+#if 0
     int vfs_current_is_extfs (void);
     int vfs_current_is_tarfs (void);
+#endif
     int vfs_file_is_local (char *name);
     int vfs_file_is_ftp (char *filename);
     char *vfs_get_current_dir (void);
@@ -142,12 +168,9 @@ struct utimbuf {
     void vfs_free_resources(char *path);
     void vfs_timeout_handler ();
     int vfs_timeouts ();
-    void vfs_force_expire (char *pathname);
 
     void vfs_fill_names (void (*)(char *));
-
-    /* Required for the vfs_canon routine */
-    char *tarfs_analysis (char *inname, char **archive, int is_dir);
+    char *vfs_translate_url (char *);
 
     void ftpfs_set_debug (char *file);
 #ifdef USE_NETCODE
@@ -160,35 +183,36 @@ struct utimbuf {
     
     /* Only the routines outside of the VFS module need the emulation macros */
 
-	int mc_open (char *file, int flags, ...);
-	int mc_close (int handle);
-	int mc_read (int handle, char *buffer, int count);
-	int mc_write (int hanlde, char *buffer, int count);
+	int   mc_open  (char *file, int flags, ...);
+	int   mc_close (int handle);
+	int   mc_read  (int handle, char *buffer, int count);
+	int   mc_write (int hanlde, char *buffer, int count);
 	off_t mc_lseek (int fd, off_t offset, int whence);
-	int mc_chdir (char *);
+	int   mc_chdir (char *);
 
-	DIR *mc_opendir (char *dirname);
+	DIR  *mc_opendir (char *dirname);
 	struct dirent *mc_readdir(DIR *dirp);
 	int mc_closedir (DIR *dir);
  	int mc_telldir (DIR *dir);
  	void mc_seekdir (DIR *dir, int offset);
 
-	int mc_stat (char *path, struct stat *buf);
+	int mc_stat  (char *path, struct stat *buf);
 	int mc_lstat (char *path, struct stat *buf);
 	int mc_fstat (int fd, struct stat *buf);
 
-	int mc_chmod (char *path, int mode);
-	int mc_chown (char *path, int owner, int group);
-	int mc_utime (char *path, struct utimbuf *times);
-	int mc_readlink(char *path, char *buf, int bufsiz);
-	int mc_unlink (char *path);
-	int mc_symlink (char *name1, char *name2);
-        int mc_link (char *name1, char *name2);
-        int mc_mknod (char *, int, int);
-	int mc_rename (char *original, char *target);
-	int mc_write (int fd, char *buf, int nbyte);
-        int mc_rmdir (char *path);
-        int mc_mkdir (char *path, mode_t mode);
+	int mc_chmod    (char *path, int mode);
+	int mc_chown    (char *path, int owner, int group);
+	int mc_utime    (char *path, struct utimbuf *times);
+	int mc_readlink (char *path, char *buf, int bufsiz);
+	int mc_unlink   (char *path);
+	int mc_symlink  (char *name1, char *name2);
+        int mc_link     (char *name1, char *name2);
+        int mc_mknod    (char *, int, int);
+	int mc_rename   (char *original, char *target);
+	int mc_write    (int fd, char *buf, int nbyte);
+        int mc_rmdir    (char *path);
+        int mc_mkdir    (char *path, mode_t mode);
+
         char *mc_getlocalcopy (char *filename);
         void mc_ungetlocalcopy (char *filename, char *local, int has_changed);
         char *mc_def_getlocalcopy (vfs *vfs, char *filename);
@@ -217,6 +241,7 @@ struct utimbuf {
 #   define vfs_current_is_tarfs() 0
 #   define vfs_current_is_extfs() 0
 #   define vfs_path(x) x
+#   define vfs_strip_suffix_from_filename (x) strdup(x)
 #   define mc_close close
 #   define mc_read read
 #   define mc_write write
@@ -257,7 +282,6 @@ struct utimbuf {
 #   define mc_rmdir rmdir
 #   define is_special_prefix(x) 0
 #   define vfs_type(x) (vfs *)(NULL)
-#   define vfs_setup_wd()
 #   define vfs_init()
 #   define vfs_shut()
 #   define vfs_canon(p) strdup (canonicalize_pathname(p))
@@ -292,34 +316,31 @@ struct utimbuf {
 
 #define mc_errno errno
 
-#ifdef WANT_PARSE_LS_LGA
-int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname);
-int split_text (char *p);
-int parse_filetype (char c);
-int parse_filemode (char *p);
-int parse_filedate(int idx, time_t *t);
-#endif
+/* These functions are meant for use by vfs modules */
+
+extern int vfs_parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname);
+extern int vfs_split_text (char *p);
+extern int vfs_parse_filetype (char c);
+extern int vfs_parse_filemode (char *p);
+extern int vfs_parse_filedate(int idx, time_t *t);
+
 extern void vfs_die (char *msg);
 extern char *vfs_get_password (char *msg);
+extern char *vfs_split_url (char *path, char **host, char **user, int *port, char **pass,
+					int default_port, int flags);
+#define URL_DEFAULTANON 1
+#define URL_NOSLASH 2
+extern void vfs_print_stats (char *fs_name, char *action, char *file_name, int have, int need);
 
-void print_vfs_stats (char *fs_name, char *action, char *file_name, int have, int need);
-
-#define MCCTL_SETREMOTECOPY	0
-#define MCCTL_ISREMOTECOPY	1
-#define MCCTL_REMOTECOPYCHUNK	2
-#define MCCTL_FINISHREMOTE	3
-#define MCCTL_FLUSHDIR          4	/* Unreferenced */
+/* Dont use values 0..4 for a while -- 10/98, pavel@ucw.cz */
 #define MCCTL_REMOVELOCALCOPY   5
 #define MCCTL_IS_NOTREADY	6
 #define MCCTL_FORGET_ABOUT	7
-
-/* Return codes from the ${fs}_ctl routine */
-
-#define MCERR_TARGETOPEN	-1    /* Can't open target file */
-#define MCERR_READ		-2    /* Read error on source file */
-#define MCERR_WRITE		-3    /* Write error on target file */
-#define MCERR_FINISH		-4    /* Finished transfer */
-#define MCERR_DATA_ON_STDIN     -5    /* Data waiting on stdin to be processed */
+#define MCCTL_EXTFS_RUN		8
+/* These two make vfs layer give out potentially incorrect data, but
+   they also make some operation 100 times faster. Use with caution. */
+#define MCCTL_WANT_STALE_DATA	9
+#define MCCTL_NO_STALE_DATA    10
 
 extern int vfs_flags;
 extern uid_t vfs_uid;
@@ -341,8 +362,6 @@ extern void mc_vfs_init( void );
 extern void mc_vfs_done( void );
 #endif
 
-#define ERRNOR(x,y) do { my_errno = x; return y; } while(0)
-
 #define O_ALL (O_CREAT | O_EXCL | O_NOCTTY | O_NDELAY | O_SYNC | O_WRONLY | O_RDWR | O_RDONLY)
 /* Midnight commander code should _not_ use other flags than those
    listed above and O_APPEND */
@@ -354,7 +373,7 @@ extern void mc_vfs_done( void );
 #define NO_LINEAR(a) a
 #else
 #define O_LINEAR O_APPEND
-#define IS_LINEAR(a) ((a) == (O_RDONLY | O_LINEAR))
+#define IS_LINEAR(a) ((a) == (O_RDONLY | O_LINEAR))	/* Return only 0 and 1 ! */
 #define NO_LINEAR(a) (((a) == (O_RDONLY | O_LINEAR)) ? O_RDONLY : (a))
 #endif
 
@@ -364,6 +383,10 @@ extern void mc_vfs_done( void );
  *     	a) to read file linearily from beggining to the end
  *	b) not to open another file before you close this one
  *		(this will likely go away in future)
+ *	as a special gift, you may
+ *	c) lseek() immediately after open(), giving ftpfs chance to
+ *	   reget. Be warned that this lseek() can fail, and you _have_
+ *         to handle that gratefully.
  *
  * O_LINEAR allows filesystems not to create temporary file in some
  * cases (ftp transfer).				-- pavel@ucw.cz
@@ -378,4 +401,32 @@ extern void mc_vfs_done( void );
 #define MMAPNULL
 #endif
 
+#define DIR_SEP_CHAR '/'
+
+/* And now some defines for our errors. */
+
+#ifdef ENOSYS
+#define E_NOTSUPP ENOSYS	/* for use in vfs when module does not provide function */
+#else
+#define E_NOTSUPP EFAULT	/* Does this happen? */
+#endif
+
+#ifdef ENOMSG
+#define E_UNKNOWN ENOMSG	/* if we do not know what error happened */
+#else
+#define E_UNKNOWN EIO	/* if we do not know what error happened */
+#endif
+
+#ifdef EREMOTEIO
+#define E_REMOTE EREMOTEIO	/* if other side of ftp/fish reports error */
+#else
+#define E_REMOTE ENETUNREACH	/* :-( there's no EREMOTEIO on some systems */
+#endif
+
+#ifdef EPROTO
+#define E_PROTO EPROTO		/* if other side fails to follow protocol */
+#else
+#define E_PROTO EIO
+#endif
+
 #endif /* __VFS_H */
